Program Primjer2;

Uses Windows, Classes, Messages, DirectDraw, SysUtils;

{$R *.RES}

Type TSprajt   = Record
                   Sirina   : integer;
                   Visina   : integer;
                   Povrsina : IDirectDrawSurface
                 End;
     TObjInfo  = Record
                    X, Y           : integer;    // Koordinate objekta
                    dX, dY         : integer;    // Pomjeraj u pravcu osa
                    PrethodniFrejm : extended;   // Vrijeme prethodnog frejma
                    FPS            : extended    // Broj frejmova u sekundi
                  End;

Const RezolucijaX          = 800;
      RezolucijaY          = 600;
      ddBPP                = 16;
      BrojFrejmovaRotacije = 20;
      BrojFrejmovaTrka     = 8;

Var WindowHandle                : HWND;
    FrejmRotacije               : integer = 1;
    FrejmTrka                   : integer = 1;
    TimerFrequency              : TLargeInteger;
    Kraj, HighResolutionCounter : Boolean;
    WindowClass                 : TWndClass;
    Poruka                      : TMsg;
    DDraw                       : IDirectDraw;
    ddPrimSurf, ddSecSurf       : IDirectDrawSurface;
    PCInfo                      : TObjInfo;
    RotacijaInfo                : TObjInfo;
    TrkInfo                     : TObjInfo;
    PCSprajt                    : TSprajt;
    RotacijaSprajt              : array [1..BrojFrejmovaRotacije] of TSprajt;
    TrkSprajt                   : array [1..BrojFrejmovaTrka] of TSprajt;

Function RGBBoja (R,G,B:integer) : longword;
  Begin
      Case ddBPP of
        16     : Result := (R shr 3) shl 11 + (G shr 2) shl 5 + B shr 3;
        24, 32 : Result := R shl 16 + G shl 8 + B
      End
  End;

Function LeadingZeros (Broj, BrojCifara : integer) : string;
  Begin
    Result := IntToStr (Broj);
    While Length (Result)<BrojCifara do
      Result := '0' + Result
  End;

Procedure PripremiTajmer;
  Begin
    HighResolutionCounter:=QueryPerformanceFrequency (TimerFrequency);
    If not HighResolutionCounter then
      TimerFrequency:=1000
  End;

Function OcitajTajmer : extended;
  Var Timer : TLargeInteger;
    Begin
      If HighResolutionCounter then
        Begin
          QueryPerformanceCounter (Timer);
          Result:=Timer/TimerFrequency
        End
                               else

        Result:=GetTickCount/TimerFrequency
    End;

Function UcitajBitmapuUPovrsinu (ImeFajla : string; var Sprajt : TSprajt) : Boolean;
  Var BitmapWidth, BitmapHeight : integer;
      hBitmap                   : HGDIObj;
      BitmapInfo                : BITMAP;
      BitmapDC, SurfaceDC       : HDC;
      SurfaceDesc               : TDDSurfaceDesc;
    Begin
      Result := False;
      
      Sprajt.Sirina := 0;
      Sprajt.Visina := 0;
      Sprajt.Povrsina := nil;

      hBitmap := LoadImage (hInstance, PChar (ImeFajla), IMAGE_BITMAP, 0, 0, LR_DEFAULTSIZE or LR_LOADFROMFILE);
      If hBitmap = 0 then
        Exit;
      GetObject (hBitmap, SizeOf (BITMAP), @BitmapInfo);
      BitmapWidth := BitmapInfo.bmWidth;
      BitmapHeight := BitmapInfo.bmHeight;
      FillChar (SurfaceDesc, SizeOf (SurfaceDesc), 0);

      SurfaceDesc.dwSize := SizeOf (SurfaceDesc);
      SurfaceDesc.dwFlags := DDSD_CAPS or DDSD_WIDTH or DDSD_HEIGHT;
      SurfaceDesc.ddsCaps.dwCaps := DDSCAPS_OFFSCREENPLAIN or DDSCAPS_SYSTEMMEMORY;
      SurfaceDesc.dwWidth := BitmapWidth;
      SurfaceDesc.dwHeight := BitmapHeight;
      
      Sprajt.Sirina := BitmapWidth;
      Sprajt.Visina := BitmapHeight;
      If Failed (DDraw.CreateSurface (SurfaceDesc, Sprajt.Povrsina, nil)) then
        Begin
          DeleteObject (hBitmap);
          Exit
        End;
      Sprajt.Povrsina.GetDC (SurfaceDC);
      BitmapDC := CreateCompatibleDC (SurfaceDC);
      SelectObject (BitmapDC, hBitmap);
      BitBlt (SurfaceDC, 0, 0, BitmapWidth, BitmapHeight, BitmapDC, 0, 0, SRCCOPY);
      Sprajt.Povrsina.ReleaseDC (SurfaceDC);
      DeleteDC (BitmapDC);
      DeleteObject (hBitmap);
      Result := True
    End;

Function UcitajSprajtove : Boolean;
  Var I           : integer;
      ColorKey    : TDDColorKey;
      PixelFormat : TDDPixelFormat;
    Begin
      Result := False;

      // Definisanje transparentne boje
      FillChar (PixelFormat, SizeOf (PixelFormat), 0);
      PixelFormat.dwSize := SizeOf (PixelFormat);
      ddPrimSurf.GetPixelFormat (PixelFormat);
      ColorKey.dwColorSpaceLowValue := PixelFormat.dwGBitMask;
      ColorKey.dwColorSpaceHighValue := PixelFormat.dwGBitMask;

      // Ucitavanje sprajtova i postavljanje transparentne boje

      If not UcitajBitmapuUPovrsinu (PWideChar (WideString ('PCLogo.bmp')), PCSprajt) then
        Exit;
      PCSprajt.Povrsina.SetColorKey (DDCKEY_SRCBLT, @ColorKey);

      For I := 1 to BrojFrejmovaRotacije do
        Begin
          If not UcitajBitmapuUPovrsinu (PWideChar (WideString ('PC-' + LeadingZeros (I,4) + '.bmp')), RotacijaSprajt [I]) then
            Exit;
          RotacijaSprajt [I].Povrsina.SetColorKey (DDCKEY_SRCBLT, @ColorKey)
        End;

      For I := 1 to BrojFrejmovaTrka do
        Begin
          If not UcitajBitmapuUPovrsinu (PWideChar (WideString ('Trk-' + LeadingZeros (I,4) + '.bmp')), TrkSprajt [I]) then
            Exit;
          TrkSprajt [I].Povrsina.SetColorKey (DDCKEY_SRCBLT, @ColorKey)
        End;

      Result := True
    End;

Function Inicijalizacija : Boolean;
  Var ddSurfDesc : TDDSurfaceDesc;
    Begin
      Result := False;

      // Kreiranje DirectDraw interfejsa
      If Failed (DirectDrawCreate (nil, DDraw, nil)) then
        Exit;

      // Postavljanje kooperativnog nivoa
      If Failed (DDraw.SetCooperativeLevel (WindowHandle, DDSCL_EXCLUSIVE or DDSCL_FULLSCREEN or DDSCL_ALLOWREBOOT)) then
        Exit;

      ShowCursor (False);

      // Podesavanje rezolucije
      If Failed (DDraw.SetDisplayMode (RezolucijaX,RezolucijaY,ddBPP)) then
        Exit;

      // Podesavanje primarne povrsine
      FillChar (ddSurfDesc, SizeOf (ddSurfDesc), 0);
      ddSurfDesc.dwSize := SizeOf (ddSurfDesc);
      ddSurfDesc.dwFlags := DDSD_CAPS or DDSD_BACKBUFFERCOUNT;
      ddSurfDesc.ddsCaps.dwCaps := DDSCAPS_PRIMARYSURFACE or DDSCAPS_COMPLEX or DDSCAPS_FLIP or DDSCAPS_VIDEOMEMORY;
      ddSurfDesc.dwBackBufferCount := 1;

      // Kreiranje primarne povrsine
      If Failed (DDraw.CreateSurface (ddSurfDesc, ddPrimSurf, nil)) then
        Exit;

      // Kreiranje sekundarne "back-buffer" povrsine
      ddSurfDesc.ddsCaps.dwCaps := DDSCAPS_BACKBUFFER;
      If Failed (ddPrimSurf.GetAttachedSurface (ddSurfDesc.ddsCaps,ddSecSurf)) then
        Exit;

      // Inicijalizacija je uspjesno zavrsena
      Result := True
    End;

Procedure IscrtajFrejm;
  Var Vrijeme  : extended;
      ddBltFX  : TDDBltFX;
      DestRect : TRect;

  Procedure IscrtajSprajt (Sprajt : TSprajt; var Info : TObjInfo);
    Begin
      ddSecSurf.BltFast (Info.X, Info.Y, Sprajt.Povrsina, nil, DDBLTFAST_DONOTWAIT or DDBLTFAST_SRCCOLORKEY)
    End;

    Begin
      If ddPrimSurf = nil then
        Exit;

      // Ocitavanje tajmera radi korektnog tajminga
      Vrijeme:=OcitajTajmer;

      // Svijetlo plava (Windows XP) boja pozadine
      ddBltFX.dwSize := SizeOf (ddBltFX);
      ddBltFX.dwFillColor := RGBBoja (58, 110, 165);

      // Brisanje ekrana
      DestRect := Rect (0, 0, RezolucijaX, RezolucijaY);
      ddSecSurf.Blt (@DestRect, nil, nil, DDBLT_WAIT or DDBLT_COLORFILL, @ddBltFX);

      // Iscrtavanje sprajtova
      IscrtajSprajt (PCSprajt, PCInfo);
      IscrtajSprajt (RotacijaSprajt [FrejmRotacije], RotacijaInfo);
      IscrtajSprajt (TrkSprajt [FrejmTrka], TrkInfo);

      // Odredjivanje sljedeceg polozaja sprajtova
      If Vrijeme - PCInfo.PrethodniFrejm >= 1 / PCInfo.FPS then
        Begin
          Inc (PCInfo.X, PCInfo.dX);
          Inc (PCInfo.Y, PCInfo.dY);
          If PCInfo.X < 0 then
            Begin
              PCInfo.X := Abs (PCInfo.X);
              PCInfo.dX := -PCInfo.dX
            End;
          If PCInfo.Y < 0 then
            Begin
              PCInfo.Y := Abs (PCInfo.Y);
              PCInfo.dY := -PCInfo.dY
            End;
          If PCInfo.X > RezolucijaX - PCSprajt.Sirina then
            Begin
              PCInfo.X := 2 * (RezolucijaX - PCSprajt.Sirina) - PCInfo.X;
              PCInfo.dX := -PCInfo.dX
            End;
          If PCInfo.Y > RezolucijaY - PCSprajt.Visina then
            Begin
              PCInfo.Y := 2 * (RezolucijaY - PCSprajt.Visina) - PCInfo.Y;
              PCInfo.dY := -PCInfo.dY
            End;

          PCInfo.PrethodniFrejm:=Vrijeme
        End;

      If Vrijeme - RotacijaInfo.PrethodniFrejm >= 1 / RotacijaInfo.FPS then
        Begin
          Inc (FrejmRotacije);
          If FrejmRotacije > BrojFrejmovaRotacije then
            FrejmRotacije := 1;

          RotacijaInfo.PrethodniFrejm:=Vrijeme
        End;

      If Vrijeme - TrkInfo.PrethodniFrejm >= 1 / TrkInfo.FPS then
        Begin
          Inc (TrkInfo.X, TrkInfo.dX);
          If TrkInfo.X >= RezolucijaX - TrkSprajt [1].Sirina then
            TrkInfo.X := 0;

          Inc (FrejmTrka);
          If FrejmTrka > BrojFrejmovaTrka then
            FrejmTrka := 1;

          TrkInfo.PrethodniFrejm:=Vrijeme
        End;

      // Zamjena primarne i sekundarne povrsine - page flipping
      ddPrimSurf.Flip (nil, DDFLIP_WAIT)
    End;

Procedure OslobodiMemoriju;
  Var I : integer;
    Begin
      PCSprajt.Povrsina := nil;
      For I := 1 to BrojFrejmovaRotacije do
        RotacijaSprajt [I].Povrsina := nil;
      For I := 1 to BrojFrejmovaTrka do
        TrkSprajt [I].Povrsina := nil
    End;

Procedure InicijalizujParametre;
  Begin
    Randomize;

    PCInfo.X := Random (RezolucijaX - PCSprajt.Sirina);
    PCInfo.Y := Random (RezolucijaY- PCSprajt.Visina);
    PCInfo.dX := 3;
    PCInfo.dY := 2;
    PCInfo.PrethodniFrejm:=0;
    PCInfo.FPS:=60;

    RotacijaInfo.X := (RezolucijaX - RotacijaSprajt [1].Sirina) div 2;
    RotacijaInfo.Y := (RezolucijaY - RotacijaSprajt [1].Visina) div 2;
    RotacijaInfo.dx := 0;
    RotacijaInfo.dY := 0;
    RotacijaInfo.PrethodniFrejm:=0;
    RotacijaInfo.FPS:=20;

    TrkInfo.X := 0;
    TrkInfo.Y := RezolucijaY - TrkSprajt [1].Visina;
    TrkInfo.dX := 7;
    TrkInfo.dY := 0;
    TrkInfo.PrethodniFrejm:=0;
    TrkInfo.FPS:=15;
  End;

Function WindowProc (Wnd:HWND;Msg,wParam,lParam: longint) : longint; StdCall;
  Begin
    Result:=0;
      Case Msg of
        WM_PAINT   : Begin
                       IscrtajFrejm;
                       ValidateRect (WindowHandle, nil)
                     End;
        WM_DESTROY : Begin
                       OslobodiMemoriju;
                       PostQuitMessage (0)
                     End;
        WM_KEYDOWN : If wParam = VK_ESCAPE then
                       DestroyWindow (WindowHandle);
        else Result:=DefWindowProc (Wnd, Msg, wParam, lParam)
      End
  End;

Procedure KreirajProzor;
  Begin
    ChDir (ExtractFilePath (ParamStr(0)));
    FillChar (WindowClass,SizeOf (WindowClass),0);
    WindowClass.hInstance:=GetModuleHandle (nil);
    With WindowClass do
      Begin
        Style:=CS_HREDRAW or CS_VREDRAW;
        hIcon:=LoadIcon (hInstance,'MAINICON');
        lpfnWndProc:=@WindowProc;
        lpszClassName:='P2Klasa'
      End;
    If Windows.RegisterClass (WindowClass)=0 then
      Begin
        MessageBox (0,'Greska u registrovanju klase.','Pomorska bitka',MB_OK);
        Exit
      End;
    WindowHandle:=CreateWindow ('P2Klasa','Pomorska bitka',WS_VISIBLE or WS_POPUP,(GetSystemMetrics (SM_CXSCREEN)-RezolucijaX) div 2,(GetSystemMetrics (SM_CYSCREEN)-RezolucijaY) div 2,RezolucijaX,RezolucijaY,0,0,WindowClass.hInstance,nil);
    If WindowHandle=0 then
      Begin
        MessageBox (WindowHandle,'Greska u kreiranju prozora.','Pomorska bitka',MB_OK);
        Exit
      End;
    ShowWindow (WindowHandle,SW_SHOWDEFAULT);
    UpdateWindow (WindowHandle)
  End;

  Begin
    DDraw := nil;

    // Provjera mogucnosti tajmera velike rezolucije
    PripremiTajmer;

    // Kreiranje prozora aplikacije
    KreirajProzor;

    // Inicijalizacija DirectX-a
    If not Inicijalizacija then
      Begin
        MessageBox (WindowHandle, 'Greska pri inicijalizaciji DirectX-a.','Greska',MB_ICONEXCLAMATION);
        Exit
      End;

    // Ucitavanje sprajtova
    If not UcitajSprajtove then
      Begin
        MessageBox (WindowHandle, 'Greska pri ucitavanju sprajtova.','Greska',MB_ICONEXCLAMATION);
        OslobodiMemoriju;
        Exit
      End;

    // Definisanje pocetnih pozicija i smjerova
    InicijalizujParametre;

    // Glavna petlja aplikacije
    Kraj:=False;
    While not Kraj do
      If PeekMessage (Poruka,0,0,0,PM_REMOVE) then
        Begin
          Kraj:=Poruka.Message=WM_QUIT;
          TranslateMessage (Poruka);
          DispatchMessage (Poruka)
        End
                                              else
        IscrtajFrejm;

    // Kraj aplikacije
    Windows.UnregisterClass ('P2Klasa', hInstance);
    ShowCursor (True)
  End.
